require 'tftp/error'

module TFTP
  module Protocol

    # Opcodes
    OP_RRQ    = 1 # Read request
    OP_WRQ    = 2 # Write request
    OP_DATA   = 3 # Data
    OP_ACK    = 4 # Acknowledgment
    OP_ERROR  = 5 # Error

    MIN_PACKET_SIZE   = 4   # Minimum packet size
    MAX_PACKET_SIZE   = 516 # Maximum packet size
    DATABLOCK_SIZE    = 512 # Size of blocks used to send files

    VALID_MODES = ['netascii', 'octet']

    # RRQ Packet
    class RRQ < BinData::Record
      bit16   :opcode
      stringz :filename
      stringz :mode
    end  

    # WRQ Packet
    class WRQ < BinData::Record
      bit16   :opcode
      stringz :filename
      stringz :mode

      def create(opts = {})
        self.opcode = OP_WRQ
        
        if opts[:filename] == nil
          raise TFTP::Error.message(:no_filename_provided)
        else
          self.filename = opts[:filename]
        end

        self.mode = opts.fetch(:mode, "octet")
        
        raise TFTP::Error.message(:unsupported_mode) if !VALID_MODES.any?{ |s| s.casecmp(self.mode)==0 }

        self
      end 
    end  

    # Data Packet
    class Data < BinData::Record
      bit16  :opcode
      bit16  :block_num
      string :data
    end

    # ACK Packet
    class ACK < BinData::Record
      bit16  :opcode
      bit16  :block_num

      def create(opts = {})
        self.opcode = OP_ACK
        self.block_num = opts.fetch(:block_num, 0)

	# Catch the invalid block nums
        raise TFTP::Error.message(:invalid_block_num) if opts[:block_num].to_i < 0

        self
      end
    end

    # Error Packet
    class Error < BinData::Record
      bit16   :opcode
      bit16   :error_code
      stringz :error_msg
    end

  end
end
